open System ;;
open System.IO ;;
open System.Globalization ;;
open System.Text.RegularExpressions ;;
open Microsoft.FSharp.Collections ;;
open System.Web ;;
open TextProcessing ;;

// Input part

type resType = | KeywData of string * string list  
               | Comment 
               | SyntError of string ;; 
let reg = Regex @"\G\s*\042([^\042]+)\042(?:\s+([^\s]+))*\s*$" ;;
let comReg = Regex @"(?:\G\s*$)|(?:\G//)" ;; 
let tildeReg = Regex @"~" ;;  
let tildeReplace str = tildeReg.Replace(str," ") ;; 
let getData str = 
  let m = reg.Match str 
  if m.Success then 
    KeywData(captureSingle m 1, 
      List.map tildeReplace (captureList m 2)) 
  else 
    let m = comReg.Match str 
    if m.Success then  
      Comment 
    else 
      SyntError str ;; 

let enString = orderString "en-US" ;;

let keyWdIn() =
  let webCat = restoreValue "webCat.bin"
  let handleLine (keywSet: Set<orderString*string>) str =
    match getData str with
    | Comment         -> keywSet
    | SyntError str   -> failwith ("Syntax error: " + str) 
    | KeywData (_,[]) -> keywSet 
    | KeywData (title,keywL) ->
        let uri = Map.find title webCat
        let addKeywd kws kw = Set.add (enString kw, uri) kws
        List.fold addKeywd keywSet keywL 
  let keyWdSet = Set.empty<orderString*string>
  fileFold handleLine keyWdSet "keywords.txt" ;;

// Output part

let preamble = 
  "<!DOCTYPE html PUBLIC \"-//W3C//DTD HTML 4.01//EN\"
   \"http://www.w3.org/TR/html4/strict.dtd\">
<html>
<head>
<title>F# Program Library Documentation Keyword Index</title>
<style type = \"text/css\">
h1 {color: purple; font-size: x-large; font-family: Verdana}
p  {font-family: Verdana; font-size: large; color: maroon}
a  {font-family: Verdana; text-decoration: none; 
    font-size: medium}
</style>
</head>
<body>
<h1>F# Program Library Documentation Keyword Index</h1>
<p>Version date: " 
   + (String.Format(CultureInfo "en-US","{0:D}",DateTime.Now))  
   + "</p>" ;;

let postamble = "</body></html>" ;;

let webOut(keyWdSet) = 
  use webPage = File.CreateText "index.html"
  let outAct oldChar (orderKwd: orderString,uri: string) = 
    let keyword = string orderKwd
    let newChar = keyword.[0]
    if (Char.ToLower newChar <> Char.ToLower oldChar
       && Char.IsLetter newChar) then  
         webPage.WriteLine "<br />"
       else ()
    webPage.Write "<a href=\"" 
    webPage.Write uri
    webPage.WriteLine "\">" 
    webPage.Write (HttpUtility.HtmlEncode keyword)
    webPage.WriteLine "</a><br />" 
    newChar 
  
  webPage.WriteLine preamble
  Set.fold outAct 'a' keyWdSet |> ignore
  webPage.WriteLine postamble
  webPage.Close()

[<EntryPoint>]
let main (param: string[]) =
  let keyWdSet = keyWdIn()
  webOut keyWdSet 
  0 ;;